searchData={"content_type":"text/markdown","items":[{"doc":"# eredis\n\nNon-blocking Redis client with focus on performance and robustness.\n\n[![Build Status](https://github.com/Nordix/eredis/workflows/CI/badge.svg)](https://github.com/Nordix/eredis)\n[![Hex pm](https://img.shields.io/hexpm/v/eredis.svg?style=flat)](https://hex.pm/packages/eredis)\n[![Hex.pm](https://img.shields.io/hexpm/dt/eredis.svg)](https://hex.pm/packages/eredis)\n\nThis fork is the official continuation of `wooga/eredis`.\nIt includes several improvements, such as\nTLS support, Sentinel support and TCP error handling corrections.\nSee [CHANGELOG.md](CHANGELOG.md) for details.\n\nNote: This client connects to a standalone Redis node, optionally using Sentinel.\nFor *Redis Cluster*, you need\n[eredis_cluster](https://github.com/Nordix/eredis_cluster) or\n[ecredis](https://github.com/HalloAppInc/ecredis).\n\nSupported Redis features:\n\n * Any command, through `eredis:q/2,3`\n * Transactions\n * Pipelining\n * Authentication & multiple DBs\n * Pubsub\n * TLS\n * Sentinel support\n\nGenerated API documentation: [doc/eredis.md](doc/eredis.md)\n\nPublished documentation can also be found on [hexdocs](https://hexdocs.pm/eredis/).","ref":"readme.html","title":"eredis","type":"extras"},{"doc":"If you have Redis running on localhost with default settings, like:\n\n    docker run --rm --net=host redis:latest\n\nyou may copy and paste the following into a shell to try out Eredis:\n\n    git clone https://github.com/Nordix/eredis.git\n    cd eredis\n    rebar3 shell\n    {ok, C} = eredis:start_link().\n    {ok,  >} = eredis:q(C, [\"SET\", \"foo\", \"bar\"]).\n    {ok,  >} = eredis:q(C, [\"GET\", \"foo\"]).\n\nTo connect to a Redis instance listening on a Unix domain socket:\n\n```erlang\n{ok, C1} = eredis:start_link({local, \"/var/run/redis.sock\"}, 0).\n```\n\nTo connect to a Redis instance using TLS:\n\n```erlang\nOptions = [{tls, [{cacertfile, \"ca.crt\"},\n                  {certfile,   \"client.crt\"},\n                  {keyfile,    \"client.key\"}]}],\n{ok, C2} = eredis:start_link(\"127.0.0.1\", ?TLS_PORT, Options),\n```\n\nTo connect to a Redis sentinel cluster:\n\n```erlang\nSentinelOptions = [{master_group, mymaster},\n                   {endpoints, [{\"127.0.0.1\", 26379}]}],\n{ok, C1} = eredis:start_link([{sentinel, SentinelOptions}]).\n```","ref":"readme.html#setup","title":"Setup - eredis","type":"extras"},{"doc":"MSET and MGET:\n\n```erlang\nKeyValuePairs = [\"key1\", \"value1\", \"key2\", \"value2\", \"key3\", \"value3\"].\n{ok,  >} = eredis:q(C, [\"MSET\" | KeyValuePairs]).\n{ok, Values} = eredis:q(C, [\"MGET\" | [\"key1\", \"key2\", \"key3\"]]).\n```\n\nHASH\n\n```erlang\nHashObj = [\"id\", \"objectId\", \"message\", \"message\", \"receiver\", \"receiver\", \"status\", \"read\"].\n{ok,  >} = eredis:q(C, [\"HMSET\", \"key\" | HashObj]).\n{ok, Values} = eredis:q(C, [\"HGETALL\", \"key\"]).\n```\n\nLIST\n\n```erlang\neredis:q(C, [\"LPUSH\", \"keylist\", \"value\"]).\neredis:q(C, [\"RPUSH\", \"keylist\", \"value\"]).\neredis:q(C, [\"LRANGE\", \"keylist\", 0, -1]).\n```\n\nTransactions:\n\n```erlang\n{ok,  >} = eredis:q(C, [\"MULTI\"]).\n{ok,  >} = eredis:q(C, [\"SET\", \"foo\", \"bar\"]).\n{ok,  >} = eredis:q(C, [\"SET\", \"bar\", \"baz\"]).\n{ok, [ >,  >]} = eredis:q(C, [\"EXEC\"]).\n```\n\nPipelining:\n\n```erlang\nP1 = [[\"SET\", a, \"1\"],\n      [\"LPUSH\", b, \"3\"],\n      [\"LPUSH\", b, \"2\"]].\n[{ok,  >}, {ok,  >}, {ok,  >}] = eredis:qp(C, P1).\n```\n\nPubsub:\n\n```erlang\n1> eredis_sub:sub_example().\nreceived {subscribed, >, }\n{ , }\n2> eredis_sub:pub_example().\nreceived {message, >, >, }\nok\n3>\n```\n\nPattern Subscribe:\n\n```erlang\n1> eredis_sub:psub_example().\nreceived {subscribed, >, }\n{ , }\n2> eredis_sub:ppub_example().\nreceived {pmessage, >, >, >, }\nok\n3>\n```","ref":"readme.html#example","title":"Example - eredis","type":"extras"},{"doc":"#","ref":"readme.html#commands","title":"Commands - eredis","type":"extras"},{"doc":"Eredis has one main function to interact with redis, which is\n`eredis:q(Client::pid(), Command::iolist())`. The response will either\nbe `{ok, Value::binary() | [binary()]}` or `{error,\nMessage::binary()}`.  The value is always the exact value returned by\nRedis, without any type conversion. If Redis returns a list of values,\nthis list is returned in the exact same order without any type\nconversion.\n\n#","ref":"readme.html#query-qp-2-3-doc-eredis-md-q-2","title":"Query: [qp/2,3](doc/eredis.md#q-2) - eredis","type":"extras"},{"doc":"To send multiple requests to redis in a batch, aka. pipelining\nrequests, you may use `eredis:qp(Client::pid(),\n[Command::iolist()])`. This function returns `{ok, [Value::binary()]}`\nwhere the values are the redis responses in the same order as the\ncommands you provided.\n\n#","ref":"readme.html#pipelined-query-qp-2-3-doc-eredis-md-qp-2","title":"Pipelined query: [qp/2,3](doc/eredis.md#qp-2) - eredis","type":"extras"},{"doc":"To start the client, use `start_link/1` or one of its variants. `start_link/1`\ntakes the following options (proplist):\n\n* `host`: DNS name or IP address as string; or unix domain socket as `{local,\n  Path}` (available in OTP 19+)\n* `port`: integer, default is 6379\n* `database`: integer or 0 for default database, default: 0\n* `username`: string, default: no username\n* `password`: string, default: no password\n* `reconnect_sleep`: integer of milliseconds to sleep between reconnect attempts, default: 100\n* `connect_timeout`: timeout value in milliseconds to use in the connect, default: 5000\n* `socket_options`: proplist of [gen_tcp](https://erlang.org/doc/man/gen_tcp.html)\n  options used when connecting the socket, default is `?SOCKET_OPTS`.\n  It's possible to specify `{active, N}` or `{active, true}`.\n  Active `once` and `false` are not supported. The default is N = 10.\n* `tls`: enable TLS by providing a list of\n  [options](https://erlang.org/doc/man/ssl.html) used when establishing the TLS\n  connection, default is off.\n  It's possible to specify `{active, N}` or `{active, true}` in TLS options.\n  Active `once` and `false` are not supported. The default is N = 10.\n  Note that `{active, N}` with TLS requires OTP 21.3 or later.","ref":"readme.html#connect-a-client-start_link-1-doc-eredis-md-start_link-1","title":"Connect a client: [start_link/1](doc/eredis.md#start_link-1) - eredis","type":"extras"},{"doc":"Commands are pipelined automatically so multiple processes can share the same\nEredis connection instance. Although `q/2,3` and `qp/2,3` are blocking until the\nresponse is returned, Eredis is not blocked.\n\n```\n  Process A          Process B          Eredis        TCP/TLS socket\n     |                  |                  |          (Redis server)\n     | q(Pid, Command1) |                  |                 |\n     |------------------------------------>|---------------->|\n     |                  | q(Pid, Command2) |                 |\n     |                  |----------------->|---------------->|\n     |                  |                  |                 |\n    ...                ...                ...               ...\n     |                  |                  |                 |\n     |                  |                  |      Response 1 |\n     |<------------------------------------|<----------------|\n     |                  |                  |      Response 2 |\n     |                  |<-----------------|<----------------|\n```","ref":"readme.html#implicit-pipelining","title":"Implicit pipelining - eredis","type":"extras"},{"doc":"When Eredis for some reason loses the connection to Redis, Eredis\nwill keep trying to reconnect until a connection is successfully\nestablished, which includes the `AUTH` and `SELECT` calls. The sleep\ntime between attempts to reconnect can be set in the\n`eredis:start_link/1` call.\n\nAs long as the connection is down, Eredis will respond to any request\nimmediately with `{error, no_connection}` without actually trying to\nconnect. This serves as a kind of circuit breaker and prevents a\nstampede of clients just waiting for a failed connection attempt or\n`gen_server:call` timeout.\n\nNote: If Eredis is starting up and cannot connect, it will fail\nimmediately with `{connection_error, Reason}`.","ref":"readme.html#reconnecting-on-redis-down-network-failure-timeout-etc","title":"Reconnecting on Redis down / network failure / timeout / etc - eredis","type":"extras"},{"doc":"Thanks to Dave Peticolas (jdavisp3), eredis supports\npubsub. [`eredis_sub`](doc/eredis_sub.md) offers a separate client that will forward\nchannel messages from Redis to an Erlang process in an \"active-once\"\npattern similar to gen_tcp sockets. After every message received, the\ncontrolling process must acknowledge receipt using\n`eredis_sub:ack_message/1`.\n\nIf the controlling process does not process messages fast enough,\neredis will queue the messages up to a certain queue size controlled\nby configuration. When the max size is reached, eredis will either\ndrop messages or crash, also based on configuration.\n\nSubscriptions are managed using `eredis_sub:subscribe/2` and\n`eredis_sub:unsubscribe/2`. When Redis acknowledges the change in\nsubscription, a message is sent to the controlling process for each\nchannel. Then, eredis sends a message on the form `{subscribed, Channel,\nClientPid}` to the controlling process, which must be acked using\n`eredis_sub:ack_message/1`.\n\neredis also supports Pattern Subscribe using `eredis_sub:psubscribe/2`\nand `eredis_sub:punsubscribe/2`. As with normal subscriptions, a message\nis sent to the controlling process for each channel and eredis sends a\nmessage on the form `{subscribed, Channel, ClientPid}` to the\ncontrolling process, which must be acked using `eredis_sub:ack_message/1`.\n\nThe controlling process is also notified in case of reconnection attempts or\nfailures. See `test/eredis_pubsub_SUITE.erl` for examples.\n\nStarting with version 1.5, eredis automatically resubscribes after reconnect.\nThen, the controlling process will receive `{subscribed, Channel, ClientPid}`\nmessages again for every channel and pattern subscribed to. These must also be\nacked.\n\nHere is a list of all the messages that are sent to the controlling process.\nSome messages don't need to be acked, but it does not harm doing so.\n\n* `{eredis_connected, Pid}` when the socket to Redis is established\n  and authenticated. Doesn't need to be acked.\n\n* `{eredis_disconnected, Pid}` when the connection to Redis has been lost.\n  Doesn't need to be acked.\n\n* `{eredis_reconnect_attempt, Pid}` at every reconnect attempt, when the\n  connection to Redis has been lost. Doesn't need to be acked.\n\n* `{eredis_reconnect_failed, Pid, {error, {connection_error, Reason}}}` after\n  every failed reconnect attempt. Doesn't need to be acked.\n\n* `{message, Channel, Message, Pid}` for every incoming message on subscribed\n  channels. Needs to be acked using `eredis_sub:ack_message/1`.\n\n* `{pmessage, Pattern, Channel, Message, Pid}` for every incoming message on\n  channels subscribed to by pattern, using using `eredis_sub:psubscribe/2`.\n  Needs to be acked using `eredis_sub:ack_message/1`.\n\n* `{subscribed, Channel, Pid}` when a subscription has been confirmed by Redis.\n  Channel is either a channel (subscribe) or a pattern (psubscribe). Needs to be\n  acked using `eredis_sub:ack_message/1`.\n\n* `{unsubscribed, Channel, Pid}` when a subscription has been removed from\n  Redis. Channel is either a channel (unsubscribe) or a pattern (punsubscribe).\n  Needs to be acked using `eredis_sub:ack_message/1`.","ref":"readme.html#pubsub","title":"Pubsub - eredis","type":"extras"},{"doc":"Eredis also implements the AUTH and SELECT calls for you. When the\nclient is started with something else than default values for password\nand database, it will issue the `AUTH` and `SELECT` commands\nappropriately, even when reconnecting after a timeout.","ref":"readme.html#auth-and-select","title":"AUTH and SELECT - eredis","type":"extras"},{"doc":"Using [lasp-bench](https://github.com/lasp-lang/lasp-bench/) you may\nbenchmark Eredis on your own hardware using the provided config and\ndriver. See `priv/basho_bench_driver_eredis.config` and\n`src/basho_bench_driver_eredis.erl`.\n\nTestcase summary from our daily runs:\n\n* [eredis](https://bjosv.github.io/eredis-benchmark/results/latest/eredis/summary.png)\n* [eredis_pipeline](https://bjosv.github.io/eredis-benchmark/results/latest/eredis_pipeline/summary.png)\n\nThe [eredis-benchmark](https://github.com/bjosv/eredis-benchmark) repo runs\na daily job that produces above graphs. It also contains the script\n`run-tests.sh` that might help you with the needed steps when setting up the\nbenchmark testing on your own.","ref":"readme.html#benchmarking","title":"Benchmarking - eredis","type":"extras"},{"doc":"Eredis uses the same queueing mechanism as Erldis. `eredis:q/2` uses\n`gen_server:call/2` to do a blocking call to the client\ngen_server. The client will immediately send the request to Redis, add\nthe caller to the queue and reply with `noreply`. This frees the\ngen_server up to accept new requests and parse responses as they come\non the socket.\n\nWhen data is received on the socket, we call `eredis_parser:parse/2`\nuntil it returns a value, we then use `gen_server:reply/2` to reply to\nthe first process waiting in the queue.\n\nThis queueing mechanism works because Redis guarantees that the\nresponse will be in the same order as the requests.","ref":"readme.html#queueing","title":"Queueing - eredis","type":"extras"},{"doc":"The socket is set to `{active, N}` with N = 10 by default. This is configurable\nby including `{active, N}` in socket options or TLS options (if TLS is used). N\nmust be an integer or `true`. Active `once` and `false` are not supported.\n\nThe response parser is capable of parsing partial data and continuing when more\ndata arrives on the socket.","ref":"readme.html#response-parsing","title":"Response parsing - eredis","type":"extras"},{"doc":"EUnit tests currently require a locally running instance of Redis.\n\n```console\nrebar3 eunit\n```\n\nXref, dialyzer and elvis should result in no errors.\n\n```console\nrebar3 xref\nrebar3 dialyzer\nelvis rock\n```","ref":"readme.html#tests-and-code-checking","title":"Tests and code checking - eredis","type":"extras"},{"doc":"When the parser is accumulating data, a new binary is generated for\nevery call to `parse/2`. This might create binaries that will be\nreference counted. This could be improved by replacing it with an\niolist.\n\nWhen parsing bulk replies, the parser knows the size of the bulk. If the\nbulk is big and would come in many chunks, this could be improved by\nhaving the client explicitly use `gen_tcp:recv/2` to fetch the entire\nbulk at once.","ref":"readme.html#future-improvements","title":"Future improvements - eredis","type":"extras"},{"doc":"This is a fork of the original Eredis. Eredis was created by Knut Nesheim, with\ninspiration from the earlier Erldis.\n\nAlthough this project is almost a complete rewrite, many patterns are\nthe same as you find in Erldis, most notably the queueing of requests.\n\n`create_multibulk/1` and `to_binary/1` were taken verbatim from Erldis.","ref":"readme.html#credits","title":"Credits - eredis","type":"extras"},{"doc":"# CHANGELOG","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"July 2023.\n\n* Allow floats in commands.\n\n* Remove unused include-lib eunit.hrl which caused problems for Elixir users.","ref":"changelog.html#v1-7-1","title":"v1.7.1 - CHANGELOG","type":"extras"},{"doc":"July 2022.\n\n* Replaces `error_logger` with `logger`.\n\n* Turns repeated reconnection errors into log level `notice`.\n\n* Parser optimizations.\n\n* Enables performance tuning of the received packets handling.\n  The socket is set to `{active, N}` with N = 10 by default.\n  This is configurable by including `{active, N}` in the socket options or\n  in the TLS options when TLS is used.\n\n  Note that `{active, N}` with TLS requires OTP 21.3 or later.\n  When using OTP below 21.3 the option needs to be set to `{active, true}`.","ref":"changelog.html#v1-7-0","title":"v1.7.0 - CHANGELOG","type":"extras"},{"doc":"July 2022.\n\n* Adds sentinel support.\n\n* Obfuscates username and password in the state, to prevent them from appearing\n  verbatim in logs and stack traces. They can be provided as 0-ary functions\n  returning the actual secret when applied.\n\n* Fix a crash that happens when reconnect fails twice with two different\n  reasons. This bug was introduced in v1.5.0.","ref":"changelog.html#v1-6-0","title":"v1.6.0 - CHANGELOG","type":"extras"},{"doc":"May 2022.\n\n* Solves a bug introduced in v1.5.0 causing the reconnect process to survive\n  if the client is stopped while attempting to reconnect.\n\n* Drops the spawned process used for reconnecting. Instead, the reconnect\n  attempts are scheduled using timers.\n\n  The option `reconnect_sleep` now applies to the time between a\n  successful connect and the first reconnect attemt, if the connection is lost\n  just after connecting. However, there is no delay before reconnecting\n  if the connection has been up for at least `reconnect_sleep` milliseconds.","ref":"changelog.html#v1-5-1","title":"v1.5.1 - CHANGELOG","type":"extras"},{"doc":"May 2022.\n\n* No delay before the first reconnect attempt.\n\n* eredis_sub: Automatic re-subscribe on reconnect. Messages on the\n  form `{subscribed, Channel, Pid}` are sent to the controlling\n  process in this case and need to be acked.\n\n* eredis_sub: TLS, custom TCP options, AUTH with username, SELECT\n  database and registered name options added.","ref":"changelog.html#v1-5-0","title":"v1.5.0 - CHANGELOG","type":"extras"},{"doc":"June 2021.\n\n* Restore support for rebar 2.\n* Optimize parser.","ref":"changelog.html#v1-4-1","title":"v1.4.1 - CHANGELOG","type":"extras"},{"doc":"June 2021.\n\n* Support for named connection processes.\n* Support AUTH with username.","ref":"changelog.html#v1-4-0","title":"v1.4.0 - CHANGELOG","type":"extras"},{"doc":"March 2021.\n\n* Include correct files in published package on hex.pm.","ref":"changelog.html#v1-3-3","title":"v1.3.3 - CHANGELOG","type":"extras"},{"doc":"February 2021.\n\n* Official release","ref":"changelog.html#v1-3-2","title":"v1.3.2 - CHANGELOG","type":"extras"},{"doc":"February 2021.\n\n* Fix build problems with Mix and other non-Rebar tools","ref":"changelog.html#v1-3-1-nordix","title":"v1.3.1-nordix - CHANGELOG","type":"extras"},{"doc":"December 2020 in the Nordix fork by Björn Svensson and Viktor Söderqvist.\n\n* TLS support\n* Correction regarding chunked error responses\n* All socket errors, including send and setopts errors, trigger reconnect\n* Explicitly close socket before reconnect, preventing dangling ports\n* Terminate reconnect loop when client terminates (fixes wooga/eredis#124)\n* Try all addresses from `inet:getaddrs/2` instead of just one when connecting\n* Stray messages don't crash the connection process\n* Improved tests and coverage\n* CI builds using Github Actions\n* Elvis code style and Dialyzer corrections","ref":"changelog.html#v1-3-0-nordix","title":"v1.3.0-nordix - CHANGELOG","type":"extras"},{"doc":"2018 in the original `wooga/eredis` repo.\n\n* Unix domain socket support (PR wooga/eredis#108 by Igor Slepchin)\n* Reset parser state on disconnect (PR wooga/eredis#106 by Ivan Baidakou)\n* Non-blocking init AKA async connect (PR wooga/eredis#105 by Valery Meleshkin)\n* Some improved tests","ref":"changelog.html#v1-2-0","title":"v1.2.0 - CHANGELOG","type":"extras"},{"doc":"* Merged a ton of of old and neglected pull requests. Thanks to\n  patient contributors:\n  * Emil Falk\n  * Evgeny Khramtsov\n  * Kevin Wilson\n  * Luis Rascão\n  * Аверьянов Илья (savonarola on github)\n  * ololoru\n  * Giacomo Olgeni\n\n* Removed rebar binary, made everything a bit more rebar3 & mix\n  friendly.","ref":"changelog.html#v1-1-0","title":"v1.1.0 - CHANGELOG","type":"extras"},{"doc":"* Fixed include directive to work with rebar 2.5.1. Thanks to Feng Hao\n  for the patch.","ref":"changelog.html#v1-0-8","title":"v1.0.8 - CHANGELOG","type":"extras"},{"doc":"* If an eredis_sub_client needs to reconnect to Redis, the controlling\n  process is now notified with the message `{eredis_reconnect_attempt,\n  Pid}`. If the reconnection attempt fails, the message is\n  `{eredis_reconnect_failed, Pid, Reason}`. Thanks to Piotr Nosek for\n  the patch.\n\n* No more deprecation warnings of the `queue` type on OTP 17. Thanks\n  to Daniel Kempkens for the patch.\n\n* Various spec fixes. Thanks to Hernan Rivas Acosta and Anton Kalyaev.","ref":"changelog.html#v1-0-7","title":"v1.0.7 - CHANGELOG","type":"extras"},{"doc":"* If the connection to Redis is lost, requests in progress will\n  receive `{error, tcp_closed}` instead of the `gen_server:call`\n  timing out. Thanks to Seth Falcon for the patch.","ref":"changelog.html#v1-0-6","title":"v1.0.6 - CHANGELOG","type":"extras"},{"doc":"* Added support for not selecting any specific database. Thanks to\n  Mikl Kurkov for the patch.","ref":"changelog.html#v1-0-5","title":"v1.0.5 - CHANGELOG","type":"extras"},{"doc":"* Added `eredis:q_noreply/2` which sends a fire-and-forget request to\n  Redis. Thanks to Ransom Richardson for the patch.\n\n* Various type annotation improvements, typo fixes and robustness\n  improvements. Thanks to Michael Gregson, Matthew Conway and Ransom\n  Richardson.","ref":"changelog.html#v1-0-4","title":"v1.0.4 - CHANGELOG","type":"extras"},{"doc":"* Fixed bug in eredis_sub where when the connection to Redis was lost,\n  the socket would not be set into {active, once} on reconnect. Thanks\n  to georgeye for the patch.","ref":"changelog.html#v1-0-3","title":"v1.0.3 - CHANGELOG","type":"extras"},{"doc":"* Fixed bug in eredis_sub where the socket was incorrectly set to\n  `{active, once}` twice. At large volumes of messages, this resulted\n  in too many messages from the socket and we would be unable to keep\n  up. Thanks to pmembrey for reporting.","ref":"changelog.html#v1-0-2","title":"v1.0.2 - CHANGELOG","type":"extras"},{"doc":"* Support added for pubsub thanks to Dave Peticolas\n  (jdavisp3). Implemented in `eredis_sub` and `eredis_sub_client` is a\n  subscriber that will forward messages from Redis to an Erlang\n  process with flow control. The user can configure to either drop\n  messages or crash the driver if a certain queue size inside the\n  driver is reached.\n\n* Fixed error handling when eredis starts up and Redis is still\n  loading the dataset into memory.","ref":"changelog.html#v1-0","title":"v1.0 - CHANGELOG","type":"extras"},{"doc":"* Support added for pipelining requests, which allows batching\n  multiple requests in a single call to eredis. Thanks to Dave\n  Peticolas (jdavisp3) for the implementation.","ref":"changelog.html#v0-7-0","title":"v0.7.0 - CHANGELOG","type":"extras"},{"doc":"* Support added for transactions, by Dave Peticolas (jdavisp3) who implemented\n  parsing of nested multibulks.","ref":"changelog.html#v0-6-0","title":"v0.6.0 - CHANGELOG","type":"extras"},{"doc":"* Configurable reconnect sleep time, by Valentino Volonghi (dialtone)\n\n* Support for using eredis as a poolboy worker, by Valentino Volonghi\n  (dialtone)","ref":"changelog.html#v0-5-0","title":"v0.5.0 - CHANGELOG","type":"extras"},{"doc":"# Module eredis #\n* [Data Types](#types)\n* [Function Index](#index)\n* [Function Details](#functions)","ref":"eredis.html","title":"Module eredis","type":"extras"},{"doc":"#","ref":"eredis.html#data-types","title":"Data Types ## - Module eredis","type":"extras"},{"doc":"client() = pid() | atom() | {atom(), atom()} | {global, term()} | {via, atom(), term()}\n \n \n\n\n\n\n  \n#","ref":"eredis.html#client","title":"client() ### - Module eredis","type":"extras"},{"doc":"host() = string() | {local, string()}\n \n \n\n\n\n\n  \n#","ref":"eredis.html#host","title":"host() ### - Module eredis","type":"extras"},{"doc":"obfuscated() = fun(() -&gt; iodata())\n \n \n\n\n\n\n  \n#","ref":"eredis.html#obfuscated","title":"obfuscated() ### - Module eredis","type":"extras"},{"doc":"option() = {host, string() | {local, string()}} | {port,  inet:port_number() } | {database, integer()} | {username, iodata() |  obfuscated()  | undefined} | {password, iodata() |  obfuscated()  | undefined} | {reconnect_sleep,  reconnect_sleep() } | {connect_timeout, integer()} | {socket_options, list()} | {tls, [ ssl:tls_client_option() ]} | {name,  registered_name() } | {sentinel, list()}\n \n \n\n\n\n\n  \n#","ref":"eredis.html#option","title":"option() ### - Module eredis","type":"extras"},{"doc":"options() = [ option() ]\n \n \n\n\n\n\n  \n#","ref":"eredis.html#options","title":"options() ### - Module eredis","type":"extras"},{"doc":"pipeline() = [iolist()]\n \n \n\n\n\n\n  \n#","ref":"eredis.html#pipeline","title":"pipeline() ### - Module eredis","type":"extras"},{"doc":"reconnect_sleep() = no_reconnect | integer()\n \n \n\n\n\n\n  \n#","ref":"eredis.html#reconnect_sleep","title":"reconnect_sleep() ### - Module eredis","type":"extras"},{"doc":"registered_name() = {local, atom()} | {global, term()} | {via, atom(), term()}\n \n \n\n\n\n\n  \n#","ref":"eredis.html#registered_name","title":"registered_name() ### - Module eredis","type":"extras"},{"doc":"return_value() = undefined | binary() | [binary() | nonempty_list()]","ref":"eredis.html#return_value","title":"return_value() ### - Module eredis","type":"extras"},{"doc":"q/2    Executes the given command in the specified connection.     q/3   Like q/2 with a custom timeout.     q_async/2   Executes the command, and sends a message to the calling process with the\nresponse (with either error or success).     q_async/3   Executes the command, and sends a message to  Pid  with the response\n(with either or success).     q_noreply/2   Executes the command but does not wait for a response and ignores any\nerrors.     qp/2    Executes the given pipeline (list of commands) in the\nspecified connection.     qp/3   Like qp/2 with a custom timeout.     start_link/0   Connect with default options.     start_link/1   Connect with the given options.     start_link/2   Connect to the given host and port.     start_link/3   ( Deprecated .)      start_link/4   ( Deprecated .)      start_link/5   ( Deprecated .)      start_link/6   ( Deprecated .)      start_link/7   ( Deprecated .)      stop/1   Closes the connection and stops the client.","ref":"eredis.html#function-index","title":"Function Index ## - Module eredis","type":"extras"},{"doc":"#","ref":"eredis.html#function-details","title":"Function Details ## - Module eredis","type":"extras"},{"doc":"q(Client:: client() , Command::[any()]) -&gt; {ok,  return_value() } | {error, Reason::binary() | no_connection}\n \n \n\n\nExecutes the given command in the specified connection. The\ncommand must be a valid Redis command and may contain arbitrary\ndata which will be converted to binaries. The returned values will\nalways be binaries.\n\n  \n\n#","ref":"eredis.html#q-2","title":"q/2 ### - Module eredis","type":"extras"},{"doc":"q(Client:: client() , Command::[any()], Timeout::timeout()) -&gt; {ok,  return_value() } | {error, Reason::binary() | no_connection}\n \n \n\n\nLike q/2 with a custom timeout.\n\n  \n\n#","ref":"eredis.html#q-3","title":"q/3 ### - Module eredis","type":"extras"},{"doc":"q_async(Client:: client() , Command::[any()]) -&gt; ok\n \n \n\n\nExecutes the command, and sends a message to the calling process with the\nresponse (with either error or success). Message is of the form `{response,\nReply}`, where `Reply` is the reply expected from `q/2`.\n\n  \n\n#","ref":"eredis.html#q_async-2","title":"q_async/2 ### - Module eredis","type":"extras"},{"doc":"q_async(Client:: client() , Command::[any()], Pid::pid() | atom()) -&gt; ok\n \n \n\n\nExecutes the command, and sends a message to `Pid` with the response\n(with either or success).\n\n__See also:__ [q_async/2](#q_async-2).\n\n  \n\n#","ref":"eredis.html#q_async-3","title":"q_async/3 ### - Module eredis","type":"extras"},{"doc":"q_noreply(Client:: client() , Command::[any()]) -&gt; ok\n \n \n\n\nExecutes the command but does not wait for a response and ignores any\nerrors.\n\n__See also:__ [q/2](#q-2).\n\n  \n\n#","ref":"eredis.html#q_noreply-2","title":"q_noreply/2 ### - Module eredis","type":"extras"},{"doc":"qp(Client:: client() , Pipeline:: pipeline() ) -&gt; [{ok,  return_value() } | {error, Reason::binary()}] | {error, no_connection}\n \n \n\n\nExecutes the given pipeline (list of commands) in the\nspecified connection. The commands must be valid Redis commands and\nmay contain arbitrary data which will be converted to binaries. The\nvalues returned by each command in the pipeline are returned in a list.\n\n  \n\n#","ref":"eredis.html#qp-2","title":"qp/2 ### - Module eredis","type":"extras"},{"doc":"qp(Client:: client() , Pipeline:: pipeline() , Timeout::timeout()) -&gt; [{ok,  return_value() } | {error, Reason::binary()}] | {error, no_connection}\n \n \n\n\nLike qp/2 with a custom timeout.\n\n  \n\n#","ref":"eredis.html#qp-3","title":"qp/3 ### - Module eredis","type":"extras"},{"doc":"start_link() -&gt; {ok, Pid::pid()} | {error, Reason::term()}\n \n \n\n\nEquivalent to [`start_link([])`](#start_link-1).\n\nConnect with default options.\n\n  \n\n#","ref":"eredis.html#start_link-0","title":"start_link/0 ### - Module eredis","type":"extras"},{"doc":"start_link(Options:: options() ) -&gt; {ok, pid()} | {error, Reason::term()}\n \n \n\n\n`Options`: \n\n  {host, Host}  \n\n\n\n DNS name or IP address as string; or unix domain\n  socket as  {local, Path}  (available in OTP 19+); default  \"127.0.0.1\" \n \n\n\n\n  {port, Port}  \n\n\n\n Integer, default is 6379\n \n\n\n\n  {database, Database}  \n\n\n\n Integer; 0 for the default database\n \n\n\n\n  {username, Username}  \n\n\n\n A 0-ary function that returns the username\n  (the preferred way to provide username as it prevents the actual secret from\n  appearing in logs and stacktraces), a string or iodata or the\n  atom  undefined  for no username; default  undefined \n \n\n\n\n  {password, Password}  \n\n\n\n A 0-ary function that returns the password\n  (the preferred way to provide password as it prevents the actual secret from\n  appearing in logs and stacktraces), a string or iodata or the\n  atom  undefined  for no password; default  undefined \n \n\n\n\n  {reconnect_sleep, ReconnectSleep}  \n\n\n\n Integer of milliseconds to\n  sleep between reconnect attempts; default: 100\n \n\n\n\n  {connect_timeout, Timeout}  \n\n\n\n Timeout value in milliseconds to use\n  when connecting to Redis; default: 5000\n \n\n\n\n  {socket_options, SockOpts}  \n\n\n\n List of gen_tcp options  used\n  when connecting the socket; default is  ?SOCKET_OPTS \n \n\n\n\n  {tls, TlsOpts}  \n\n\n\n Enabling TLS and a list of ssl options ; used when\n  establishing a TLS connection; default is off\n \n\n\n\n  {name, Name}  \n\n\n\n Tuple to register the client with a name\n  such as  {local, atom()} ; for all options see  ServerName  at gen_server:start_link/4 ;\n  default: no name\n \n\n\n\n  {sentinel, SentinelOptions}  \n\n\n\n  Enables client for sentinel mode; options are required\n  to connect to sentinel cluster; default: undefined \n \n\n\n\n  Sentinel Options:  {master_group, master_group_name}  - Atom, default: mymaster; {endpoints, [{Host, Port}]}  - List of {Host, Port} tuples, default: [{\"127.0.0.1\", 26379}]; {username, Username} ; {password, Password} ; {connect_timeout, Timeout} ; {socket_options, SockOpts}  {tls, TlsOpts} \n \n\n\n\nConnect with the given options.\n\n  \n\n#","ref":"eredis.html#start_link-1","title":"start_link/1 ### - Module eredis","type":"extras"},{"doc":"start_link(Host:: host() , Port:: inet:port_number() ) -&gt; {ok, Pid::pid()} | {error, Reason::term()}\n \n \n\n\nEquivalent to [`start_link([{host, Host}, {port, Port}])`](#start_link-1).\n\nConnect to the given host and port.\n\n  \n\n#","ref":"eredis.html#start_link-2","title":"start_link/2 ### - Module eredis","type":"extras"},{"doc":"start_link(Host:: host() , Port:: inet:port_number() , OptionsOrDatabase) -&gt; {ok, Pid::pid()} | {error, Reason::term()}\n \n \n\n   OptionsOrDatabase =  options()  | integer()   \n\n__This function is deprecated:__ Use [`start_link/1`](#start_link-1) instead.\n\n  \n\n#","ref":"eredis.html#start_link-3","title":"start_link/3 ### - Module eredis","type":"extras"},{"doc":"start_link(Host:: host() , Port:: inet:port_number() , Database::integer(), Password::string()) -&gt; {ok, pid()} | {error, term()}\n \n \n\n\n__This function is deprecated:__ Use [`start_link/1`](#start_link-1) instead.\n\n__See also:__ [start_link/1](#start_link-1).\n\n  \n\n#","ref":"eredis.html#start_link-4","title":"start_link/4 ### - Module eredis","type":"extras"},{"doc":"start_link(Host:: host() , Port:: inet:port_number() , Database::integer(), Password::string(), ReconnectSleep:: reconnect_sleep() ) -&gt; {ok, pid()} | {error, term()}\n \n \n\n\n__This function is deprecated:__ Use [`start_link/1`](#start_link-1) instead.\n\n__See also:__ [start_link/1](#start_link-1).\n\n  \n\n#","ref":"eredis.html#start_link-5","title":"start_link/5 ### - Module eredis","type":"extras"},{"doc":"start_link(Host:: host() , Port:: inet:port_number() , Database::integer(), Password::string(), ReconnectSleep:: reconnect_sleep() , ConnectTimeout::timeout()) -&gt; {ok, pid()} | {error, term()}\n \n \n\n\n__This function is deprecated:__ Use [`start_link/1`](#start_link-1) instead.\n\n__See also:__ [start_link/1](#start_link-1).\n\n  \n\n#","ref":"eredis.html#start_link-6","title":"start_link/6 ### - Module eredis","type":"extras"},{"doc":"start_link(Host:: host() , Port:: inet:port_number() , Database::integer(), Password::string(), ReconnectSleep:: reconnect_sleep() , ConnectTimeout::timeout(), SocketOptions::list()) -&gt; {ok, pid()} | {error, term()}\n \n \n\n\n__This function is deprecated:__ Use [`start_link/1`](#start_link-1) instead.\n\n__See also:__ [start_link/1](#start_link-1).\n\n  \n\n#","ref":"eredis.html#start_link-7","title":"start_link/7 ### - Module eredis","type":"extras"},{"doc":"stop(Client:: client() ) -&gt; ok\n \n \n\n\nCloses the connection and stops the client.","ref":"eredis.html#stop-1","title":"stop/1 ### - Module eredis","type":"extras"},{"doc":"# Module eredis_sub #\n* [Data Types](#types)\n* [Function Index](#index)\n* [Function Details](#functions)","ref":"eredis_sub.html","title":"Module eredis_sub","type":"extras"},{"doc":"#","ref":"eredis_sub.html#data-types","title":"Data Types ## - Module eredis_sub","type":"extras"},{"doc":"channel() = binary()\n \n \n\n\n\n\n  \n#","ref":"eredis_sub.html#channel","title":"channel() ### - Module eredis_sub","type":"extras"},{"doc":"obfuscated() = fun(() -&gt; iodata())\n \n \n\n\n\n\n  \n#","ref":"eredis_sub.html#obfuscated","title":"obfuscated() ### - Module eredis_sub","type":"extras"},{"doc":"option() = {host, string() | {local, string()}} | {port,  inet:port_number() } | {database, integer()} | {username, iodata() |  obfuscated()  | undefined} | {password, iodata() |  obfuscated()  | undefined} | {reconnect_sleep,  reconnect_sleep() } | {connect_timeout, integer()} | {socket_options, list()} | {tls, [ ssl:tls_client_option() ]} | {name,  registered_name() } | {sentinel, list()}\n \n \n\n\n\n\n  \n#","ref":"eredis_sub.html#option","title":"option() ### - Module eredis_sub","type":"extras"},{"doc":"reconnect_sleep() = no_reconnect | integer()\n \n \n\n\n\n\n  \n#","ref":"eredis_sub.html#reconnect_sleep","title":"reconnect_sleep() ### - Module eredis_sub","type":"extras"},{"doc":"registered_name() = {local, atom()} | {global, term()} | {via, atom(), term()}\n \n \n\n\n\n\n  \n#","ref":"eredis_sub.html#registered_name","title":"registered_name() ### - Module eredis_sub","type":"extras"},{"doc":"sub_option() = {max_queue_size, integer() | infinity} | {queue_behaviour, drop | exit} |  option() \n \n \n\n\n\n\n  \n#","ref":"eredis_sub.html#sub_option","title":"sub_option() ### - Module eredis_sub","type":"extras"},{"doc":"sub_options() = [ sub_option() ]","ref":"eredis_sub.html#sub_options","title":"sub_options() ### - Module eredis_sub","type":"extras"},{"doc":"ack_message/1    acknowledge the receipt of a pubsub message.     channels/1    Returns the channels the given client is currently\nsubscribing to.     controlling_process/1    Make the calling process the controlling process.     controlling_process/2    Make the given process (pid) the controlling process.     controlling_process/3    Make the given process (pid) the controlling process subscriber\nwith the given Timeout.     psubscribe/2    Pattern subscribe to the given channels.     punsubscribe/2        start_link/0        start_link/1   Start with options in proplist format.     start_link/3   ( Deprecated .)      start_link/6   ( Deprecated .)      stop/1        subscribe/2    Subscribe to the given channels.     unsubscribe/2","ref":"eredis_sub.html#function-index","title":"Function Index ## - Module eredis_sub","type":"extras"},{"doc":"#","ref":"eredis_sub.html#function-details","title":"Function Details ## - Module eredis_sub","type":"extras"},{"doc":"ack_message(Client::pid()) -&gt; ok\n \n \n\n\nacknowledge the receipt of a pubsub message. each pubsub\nmessage must be acknowledged before the next one is received\n\n  \n\n#","ref":"eredis_sub.html#ack_message-1","title":"ack_message/1 ### - Module eredis_sub","type":"extras"},{"doc":"`channels(Client) -> any()`\n\nReturns the channels the given client is currently\nsubscribing to. Note: this list is based on the channels at startup\nand any channel added during runtime. It might not immediately\nreflect the channels Redis thinks the client is subscribed to.\n\n  \n\n#","ref":"eredis_sub.html#channels-1","title":"channels/1 ### - Module eredis_sub","type":"extras"},{"doc":"controlling_process(Client::pid()) -&gt; ok\n \n \n\n\nMake the calling process the controlling process. The\ncontrolling process received pubsub-related messages, of which\nthere are three kinds. In each message, the pid refers to the\neredis client process.\n\n{message, Channel::binary(), Message::binary(), pid()}\nThis is sent for each pubsub message received by the client.\n\n{pmessage, Pattern::binary(), Channel::binary(), Message::binary(), pid()}\nThis is sent for each pattern pubsub message received by the client.\n\n{dropped, NumMessages::integer(), pid()}\nIf the queue reaches the max size as specified in start_link\nand the behaviour is to drop messages, this message is sent when\nthe queue is flushed.\n\n{subscribed, Channel::binary(), pid()}\nWhen using eredis_sub:subscribe(pid()), this message will be\nsent for each channel Redis aknowledges the subscription. The\nopposite, 'unsubscribed' is sent when Redis aknowledges removal\nof a subscription.\n\n{eredis_disconnected, pid()}\nThis is sent when the eredis client is disconnected from redis.\n\n{eredis_connected, pid()}\nThis is sent when the eredis client reconnects to redis after\nan existing connection was disconnected.\n\nAny message of the form {message, _, _, _} must be acknowledged\nbefore any subsequent message of the same form is sent. This\nprevents the controlling process from being overrun with redis\npubsub messages. See ack_message/1.\n\n  \n\n#","ref":"eredis_sub.html#controlling_process-1","title":"controlling_process/1 ### - Module eredis_sub","type":"extras"},{"doc":"controlling_process(Client::pid(), Pid::pid()) -&gt; ok\n \n \n\n\nMake the given process (pid) the controlling process.\n\n  \n\n#","ref":"eredis_sub.html#controlling_process-2","title":"controlling_process/2 ### - Module eredis_sub","type":"extras"},{"doc":"`controlling_process(Client, Pid, Timeout) -> any()`\n\nMake the given process (pid) the controlling process subscriber\nwith the given Timeout.\n\n  \n\n#","ref":"eredis_sub.html#controlling_process-3","title":"controlling_process/3 ### - Module eredis_sub","type":"extras"},{"doc":"psubscribe(Client::pid(), Channels::[ channel() ]) -&gt; ok\n \n \n\n\nPattern subscribe to the given channels. Returns immediately. The\nresult will be delivered to the controlling process as any other\nmessage. Delivers {subscribed, Channel::binary(), pid()}\n\n  \n\n#","ref":"eredis_sub.html#psubscribe-2","title":"psubscribe/2 ### - Module eredis_sub","type":"extras"},{"doc":"punsubscribe(Client::pid(), Channels::[ channel() ]) -&gt; ok\n \n \n\n\n  \n\n#","ref":"eredis_sub.html#punsubscribe-2","title":"punsubscribe/2 ### - Module eredis_sub","type":"extras"},{"doc":"`start_link() -> any()`\n\n  \n\n#","ref":"eredis_sub.html#start_link-0","title":"start_link/0 ### - Module eredis_sub","type":"extras"},{"doc":"start_link(Options:: sub_options() ) -&gt; {ok, Pid::pid()} | {error, Reason::term()}\n \n \n\n\n`Options`: \n\n  {host, Host}  \n\n\n\n DNS name or IP address as string; or unix domain\n  socket as  {local, Path}  (available in OTP 19+); default  \"127.0.0.1\" \n \n\n\n\n  {port, Port}  \n\n\n\n Integer, default is 6379\n \n\n\n\n  {database, Database}  \n\n\n\n Integer; 0 for the default database\n \n\n\n\n  {username, Username}  \n\n\n\n A 0-ary function that returns the username\n  (the preferred way to provide username as it prevents the actual secret from\n  appearing in logs and stacktraces), a string or iodata or the atom undefined  for no username; default  undefined \n \n\n\n\n  {password, Password}  \n\n\n\n A 0-ary function that returns the password\n  (the preferred way to provide password as it prevents the actual secret from\n  appearing in logs and stacktraces), a string or iodata or the atom undefined  for no username; default  undefined \n \n\n\n\n  {reconnect_sleep, ReconnectSleep}  \n\n\n\n Integer of milliseconds to\n  sleep between reconnect attempts; default: 100\n \n\n\n\n  {connect_timeout, Timeout}  \n\n\n\n Timeout value in milliseconds to use\n  when connecting to Redis; default: 5000\n \n\n\n\n  {socket_options, SockOpts}  \n\n\n\n List of gen_tcp options  used\n  when connecting the socket; default is  ?SOCKET_OPTS \n \n\n\n\n  {tls, TlsOpts}  \n\n\n\n Enabling TLS and a list of ssl options ; used when\n  establishing a TLS connection; default is off\n \n\n\n\n  {name, Name}  \n\n\n\n Tuple to register the client with a name\n  such as  {local, atom()} ; for all options see  ServerName  at gen_server:start_link/4 ;\n  default: no name\n \n\n\n\n  {max_queue_size, N}  \n\n\n\n Queue size for incoming pubsub messages\n \n\n\n\n  {queue_behaviour, drop | exit}  \n\n\n\n What to do if the controlling\n  process doesn't ack pubsub messages fast enough\n \n\n.\n\nStart with options in proplist format.\n\n  \n\n#","ref":"eredis_sub.html#start_link-1","title":"start_link/1 ### - Module eredis_sub","type":"extras"},{"doc":"`start_link(Host, Port, Password) -> any()`\n\n__This function is deprecated:__ Use [`start_link/1`](#start_link-1) instead.\n\n  \n\n#","ref":"eredis_sub.html#start_link-3","title":"start_link/3 ### - Module eredis_sub","type":"extras"},{"doc":"`start_link(Host, Port, Password, ReconnectSleep, MaxQueueSize, QueueBehaviour) -> any()`\n\n__This function is deprecated:__ Use [`start_link/1`](#start_link-1) instead.\n\n  \n\n#","ref":"eredis_sub.html#start_link-6","title":"start_link/6 ### - Module eredis_sub","type":"extras"},{"doc":"`stop(Pid) -> any()`\n\n  \n\n#","ref":"eredis_sub.html#stop-1","title":"stop/1 ### - Module eredis_sub","type":"extras"},{"doc":"subscribe(Client::pid(), Channels::[ channel() ]) -&gt; ok\n \n \n\n\nSubscribe to the given channels. Returns immediately. The\nresult will be delivered to the controlling process as any other\nmessage. Delivers {subscribed, Channel::binary(), pid()}\n\n  \n\n#","ref":"eredis_sub.html#subscribe-2","title":"subscribe/2 ### - Module eredis_sub","type":"extras"},{"doc":"unsubscribe(Client::pid(), Channels::[ channel() ]) -&gt; ok","ref":"eredis_sub.html#unsubscribe-2","title":"unsubscribe/2 ### - Module eredis_sub","type":"extras"}]}